@@ -46,15 +46,16 @@ class DeployerTests(unittest.TestCase):
     def test_load(self):
         d = Deployment(juju_env='gojuju')
         schema = '{"mybundle": {"series": "raring", "services": {"wordpress": \
-                  {"branch": "lp:~charmers/charms/precise/wordpress/trunk"}, \
-                  "mysql": {"options": {"tuning": "fastest"}, \
-                  "constraints": "mem=2G cpu-cores=2", \
+                  {"branch": "lp:~charmers/charms/precise/wordpress/trunk", \
+                  "expose": true}, "mysql": {"options": {"tuning": \
+                  "fastest"}, "constraints": "mem=2G cpu-cores=2", \
                   "branch": "lp:~charmers/charms/precise/mysql/trunk"}}, \
                   "relations": [["mysql:db", "wordpress:db"]]}}'
         dmap = json.loads(schema)
         with patch.object(d, 'add') as add:
             with patch.object(d, 'configure') as configure:
-                d.load(dmap)
+                with patch.object(d, 'expose') as expose:
+                    d.load(dmap)
         self.assertEqual(d.juju_env, 'gojuju')
         self.assertEqual(dmap['mybundle']['relations'], d.relations)
         self.assertEqual(dmap['mybundle']['series'], d.series)
@@ -78,6 +79,9 @@ class DeployerTests(unittest.TestCase):
         configure.assert_has_calls([
             call('mysql', {'tuning': 'fastest'}),
         ])
+        expose.assert_has_calls([
+            call('wordpress'),
+        ])
 
     @patch('amulet.charm.CharmCache.get_charm')
     def test_add(self, mcharm):
@@ -145,10 +149,12 @@ class DeployerTests(unittest.TestCase):
                     total_units += 1
                     status['services'][service]['units'][
                         '{}/{}'.format(service, unit)] = {
+                        'agent-state': 'started',
                         'public-address': '10.0.3.{}'.format(total_units)}
             status['services']['relation-sentry'] = {
                 'units': {
                     'relation-sentry/0': {
+                        'agent-state': 'started',
                         'public-address': '10.0.3.1'}}}
             return status
         return _mock_status
@@ -179,6 +185,36 @@ class DeployerTests(unittest.TestCase):
         self.assertTrue('charm/1' in d.sentry.unit)
         self.assertEqual(2, d.services['charm']['num_units'])
 
+    @patch.object(UnitSentry, 'upload_scripts')
+    @patch('amulet.helpers.environments')
+    @patch('amulet.sentry.waiter.status')
+    @patch('amulet.deployer.subprocess')
+    @patch('amulet.charm.CharmCache.get_charm')
+    def test_add_unit_target(self, mcharm, subprocess, waiter_status,
+                             environments, upload_scripts):
+        """
+        If target is given to Deployment.add_unit(), the 'juju' call
+        will includde --to to make the unit be added to the given
+        target.
+        """
+        charm = mcharm.return_value
+        charm.subordinate = False
+        charm.code_source = {'location': 'lp:charms/charm'}
+        charm.url = None
+        charm.series = 'precise'
+
+        environments.return_value = yaml.safe_load(RAW_ENVIRONMENTS_YAML)
+
+        d = Deployment(juju_env='gojuju')
+
+        waiter_status.side_effect = self._make_mock_status(d)
+        d.add('charm', units=1)
+        d.setup()
+        with patch('amulet.deployer.juju') as j:
+            d.add_unit('charm', target='lxc:0')
+            j.assert_called_with(
+                ['add-unit', 'charm', '-n', '1', '--to', 'lxc:0'])
+
     @patch.object(UnitSentry, 'upload_scripts')
     @patch('amulet.helpers.environments')
     @patch('amulet.sentry.waiter.status')
@@ -424,6 +460,71 @@ class DeployerTests(unittest.TestCase):
             self.assertEqual(
                 str(e), 'All relations must be explicit, service:relation')
 
+    @patch('amulet.deployer.juju')
+    def test_action_defined(self, mj):
+        mj.return_value = '{"action": "description"}'
+        d = Deployment(juju_env='gojuju')
+        d.add('mysql')
+        actions = d.action_defined('mysql')
+        self.assertEquals(actions, {"action": "description"})
+        mj.assert_has_calls([call(['action', 'defined', 'mysql', '--format', 'json'])])
+
+    @patch('amulet.deployer.juju')
+    def test_action_do(self, mj):
+        mj.return_value = '{"Action queued with id": "some-action-id"}'
+        d = Deployment(juju_env='gojuju')
+        d.add('mysql')
+        uuid = d.action_do('mysql/0', 'run')
+        self.assertEquals(uuid, "some-action-id")
+        mj.assert_has_calls([call(['action', 'do', 'mysql/0', 'run', '--format', 'json'])])
+
+    @patch('amulet.deployer.juju')
+    def test_action_fetch(self, mj):
+        mj.side_effect = ['{"Action queued with id": "some-action-id"}',
+                          '{"results":{"key":"value"},"status":"completed",'\
+                          '"timing":{"completed":"2015-07-21 09:05:11 +0300 EEST",'\
+                          '"enqueued":"2015-07-21 09:05:06 +0300 EEST",'\
+                          '"started":"2015-07-21 09:05:09 +0300 EEST"}}']
+        d = Deployment(juju_env='gojuju')
+        d.add('mysql')
+        uuid = d.action_do('mysql/0', 'run', {"action_param": "action_value"})
+        self.assertEquals(uuid, "some-action-id")
+        results = d.action_fetch(uuid)
+        self.assertEquals(results, {'key': 'value'})
+        mj.assert_has_calls([call(['action', 'do', 'mysql/0', 'run', '--format', 'json', 'action_param=action_value']),
+                         call(['action', 'fetch', 'some-action-id', '--format', 'json', '--wait', '600'])])
+
+    @patch('amulet.deployer.juju')
+    def test_action_fetch_nowait_fail(self, mj):
+        mj.side_effect = ['{"Action queued with id": "some-action-id"}',
+                          '{"status":"running",'\
+                          '"timing":{"enqueued":"2015-07-21 09:50:59 +0300 EEST",'\
+                          '"started":"2015-07-21 09:51:04 +0300 EEST"}}']
+        d = Deployment(juju_env='gojuju')
+        d.add('mysql')
+        uuid = d.action_do('mysql/0', 'run')
+        self.assertEquals(uuid, "some-action-id")
+        results = d.action_fetch(uuid, timeout=None)
+        self.assertEquals(results, {})
+        mj.assert_has_calls([call(['action', 'do', 'mysql/0', 'run', '--format', 'json']),
+                             call(['action', 'fetch', 'some-action-id', '--format', 'json'])])
+
+    @patch('amulet.deployer.juju')
+    def test_action_fetch_wait(self, mj):
+        mj.side_effect = ['{"Action queued with id": "some-action-id"}',
+                          '{"results":{"key":"value"},"status":"completed",'\
+                          '"timing":{"completed":"2015-07-21 09:05:11 +0300 EEST",'\
+                          '"enqueued":"2015-07-21 09:05:06 +0300 EEST",'\
+                          '"started":"2015-07-21 09:05:09 +0300 EEST"}}']
+        d = Deployment(juju_env='gojuju')
+        d.add('mysql')
+        uuid = d.action_do('mysql/0', 'run')
+        self.assertEquals(uuid, "some-action-id")
+        results = d.action_fetch(uuid)
+        self.assertEquals(results, {'key': 'value'})
+        mj.assert_has_calls([call(['action', 'do', 'mysql/0', 'run', '--format', 'json']),
+                             call(['action', 'fetch', 'some-action-id', '--format', 'json', '--wait', '600'])])
+        
     @patch('amulet.deployer.juju')
     def test_unrelate(self, mj):
         d = Deployment(juju_env='gogo')
